-----------------------------------------------------------------------
-- Locale
--

local l = GetLocale()

local TOO_MANY = "Too many results, %d not shown. Whisper me !next to get 5 more."
local PREFIX = "cg: "
local CG_USAGE = "Whisper me '!%s <anything>' to search."
local NO_DATA = "No results found for %q."
local NO_SKILL = "I don't know this skill. Accepted commands are %s."
local ACCEPTED = "Accepted commands are %s."

-- All tradeskills ninjaed from Babble-Tradeskill-2.2
local skills = nil
if l == "enUS" then
	skills = {
		["Alchemy"] = true,
		["Blacksmithing"] = true,
		["Cooking"] = true,
		["Enchanting"] = true,
		["Engineering"] = true,
		["First Aid"] = true,
		["Jewelcrafting"] = true,
		["Leatherworking"] = true,
		["Tailoring"] = true,
	}
	for k, v in pairs(skills) do
		skills[k] = k
	end
elseif l == "deDE" then
	skills = {
		["Alchemy"] = "Alchimie",
		["Blacksmithing"] = "Schmiedekunst",
		["Cooking"] = "Kochkunst",
		["Enchanting"] = "Verzauberkunst",
		["Engineering"] = "Ingenieurskunst",
		["First Aid"] = "Erste Hilfe",
		["Jewelcrafting"] = "Juwelenschleifen",
		["Leatherworking"] = "Lederverarbeitung",
		["Tailoring"] = "Schneiderei",
	}
elseif l == "frFR" then
	skills = {
		["Alchemy"] = "Alchimie",
		["Blacksmithing"] = "Forge",
		["Cooking"] = "Cuisine",
		["Enchanting"] = "Enchantement",
		["Engineering"] = "Ingnierie",
		["First Aid"] = "Premiers soins",
		["Jewelcrafting"] = "Joaillerie",
		["Leatherworking"] = "Travail du cuir",
		["Tailoring"] = "Couture",
	}
elseif l == "zhCN" then
	skills = {
		["Alchemy"] = "炼金术",
		["Blacksmithing"] = "锻造",
		["Cooking"] = "烹饪",
		["Enchanting"] = "附魔",
		["Engineering"] = "工程学",
		["First Aid"] = "急救",
		["Jewelcrafting"] = "珠宝加工",	--TBC check
		["Leatherworking"] = "制皮",
		["Tailoring"] = "裁缝",
	}
elseif l == "zhTW" then
	skills = {
		["Alchemy"] = "鍊金術",
		["Blacksmithing"] = "鍛造",
		["Cooking"] = "烹飪",
		["Enchanting"] = "附魔",
		["Engineering"] = "工程學",
		["First Aid"] = "急救",
		["Jewelcrafting"] = "珠寶設計",
		["Leatherworking"] = "製皮",
		["Tailoring"] = "裁縫",
	}
elseif l == "koKR" then
	skills = {
		["Alchemy"] = "연금술",
		["Blacksmithing"] = "대장기술",
		["Cooking"] = "요리",
		["Enchanting"] = "마법부여",
		["Engineering"] = "기계공학",
		["First Aid"] = "응급치료",
		["Jewelcrafting"] = "보석세공",
		["Leatherworking"] = "가죽세공",
		["Tailoring"] = "재봉술",
	}
elseif l == "esES" then
	skills = {
		["Alchemy"] = "Alquimia",
		["Blacksmithing"] = "Herreria",
		["Cooking"] = "Cocina",
		["Enchanting"] = "Encantamiento",
		["Engineering"] = "Ingenieria",
		["First Aid"] = "Primeros auxilios",
		["Jewelcrafting"] = "Joyeria",
		["Leatherworking"] = "Peleteria",
		["Tailoring"] = "Sastreria",
	}
end

-----------------------------------------------------------------------
-- Tooltip scanner, mostly stolen from Gratuity.
--

local scanItem
do
	local scanner = CreateFrame("GameTooltip")
	scanner:SetOwner(WorldFrame, "ANCHOR_NONE")
	local lcache, rcache = {}, {}
	for i = 1, 30 do
		lcache[i], rcache[i] = scanner:CreateFontString(), scanner:CreateFontString()
		lcache[i]:SetFontObject(GameFontNormal); rcache[i]:SetFontObject(GameFontNormal)
		scanner:AddFontStrings(lcache[i], rcache[i])
	end
	local function erase()
		for i = 1, scanner:NumLines() do rcache[i]:SetText() end
		scanner:ClearLines()
	end
	local scannerCache = {}
	function scanItem(itemLink)
		erase()
		scanner:SetHyperlink(itemLink)
		for k in pairs(scannerCache) do scannerCache[k] = nil end
		for i = 1, scanner:NumLines() do
			local left, right = lcache[i]:GetText(), rcache[i]:GetText()
			if left then table.insert(scannerCache, left) end
			if right then table.insert(scannerCache, right) end
		end
		return table.concat(scannerCache)
	end
end

-----------------------------------------------------------------------
-- CraftyGnome
--

local bangMappings = {
	alchemy = skills["Alchemy"],
	blacksmith = skills["Blacksmithing"],
	cooking = skills["Cooking"],
	leather = skills["Leatherworking"],
	enchanting = skills["Enchanting"],
	engineering = skills["Engineering"],
	firstaid = skills["First Aid"],
	jewel = skills["Jewelcrafting"],
	tailor = skills["Tailoring"],
}
bangMappings.alc = bangMappings.alchemy
bangMappings.bs = bangMappings.blacksmith
bangMappings.cook = bangMappings.cooking
bangMappings.lw = bangMappings.leather
bangMappings.enchant = bangMappings.enchanting
bangMappings.eng = bangMappings.engineering
bangMappings.aid = bangMappings.firstaid
bangMappings.jc = bangMappings.jewel
bangMappings.gem = bangMappings.jewel
bangMappings.tailoring = bangMappings.tailor
bangMappings.craft = true
bangMappings.cg = true
bangMappings.next = true

local MAX_RESULTS = 5
local db = nil
local cache = {}
local acceptedCommands = {}
local addon = CreateFrame("Frame")

function addon:CRAFT_SHOW()
	local name = GetCraftName()
	if not db[name] then db[name] = {} end
	for i = 1, GetNumCrafts() do
		local skillName, _, skillType, _, _, _, level = GetCraftInfo(i)
		if skillType ~= "header" and skillName then
			local recipe = GetCraftRecipeLink(i)
			local l = level or 1
			if not db[name][recipe] or db[name][recipe] < l then
				db[name][recipe] = l
			end
		end
	end
end

function addon:TRADE_SKILL_SHOW()
	local name = GetTradeSkillLine()
	if not db[name] then db[name] = {} end
	for i = 1, GetNumTradeSkills() do
		local skillName, skillType = GetTradeSkillInfo(i)
		if skillType ~= "header" and skillName then
			local _, _, _, level = GetItemInfo(GetTradeSkillItemLink(i))
			local recipe = GetTradeSkillRecipeLink(i)
			local l = level or 1
			if not db[name][recipe] or db[name][recipe] < l then
				db[name][recipe] = l
			end
		end
	end
end

do
	local ctl = ChatThrottleLib
	local reply = ctl and
		function(msg, player) ctl:SendChatMessage("BULK", "CraftyGnome", PREFIX .. msg, "WHISPER", nil, player) end or
		function(msg, player) SendChatMessage(PREFIX .. msg, "WHISPER", nil, player) end

	local function updateCache(skill)
		if not cache[skill] then cache[skill] = {} end
		for item in pairs(db[skill]) do
			if not cache[skill][item] then
				cache[skill][item] = scanItem(item):lower()
			end
		end
	end

	local function updateAcceptedCommands()
		for k in pairs(acceptedCommands) do acceptedCommands[k] = nil end
		for k, v in pairs(bangMappings) do
			if db[v] then
				table.insert(acceptedCommands, "!" .. k)
			end
		end
	end

	local moreResults = {}
	local skillSorters = {}
	local function processChat(msg, player, isWhisper)
		local cmd = select(3, msg:find("^!(%S+)"))
		if not cmd or not bangMappings[cmd] then return end
		if cmd == "next" then
			if moreResults[player] then
				for i = 1, MAX_RESULTS do
					local r = moreResults[player][i]
					if r then reply(r, player) end
				end
				for i = 1, MAX_RESULTS do table.remove(moreResults[player], 1) end
				if #moreResults[player] > 0 then
					reply(TOO_MANY:format(#moreResults[player]), player)
				else
					moreResults[player] = nil
				end
			end
			return
		end
		if type(bangMappings[cmd]) == "boolean" then
			updateAcceptedCommands()
			reply(ACCEPTED:format(table.concat(acceptedCommands, ", ")), player)
			return
		end
		if moreResults[player] then moreResults[player] = nil end
		local query = select(3, msg:find("^!%S+%s(.*)"))
		if query then
			local skill = bangMappings[cmd]
			if db[skill] then
				updateCache(skill)
				local queryTokens = nil
				local name = GetItemInfo(query)
				-- This is an item, don't split it up into multiple tokens
				if name then
					queryTokens = {name}
				else
					queryTokens = {strsplit(" ", query)}
				end
				for i, v in ipairs(queryTokens) do queryTokens[i] = v:lower() end
				local tbl = nil
				if cache[skill] then
					for item, details in pairs(cache[skill]) do
						local matchAll = true
						for i, token in ipairs(queryTokens) do
							if not details:find(token) then
								matchAll = nil
							end
						end
						if matchAll then
							if not tbl then tbl = {} end
							table.insert(tbl, item)
						end
					end
				end
				queryTokens = nil
				if tbl then
					if not skillSorters[skill] then
						skillSorters[skill] = function(a, b)
							return db[skill][a] > db[skill][b]
						end
					end
					table.sort(tbl, skillSorters[skill])
					if #tbl > MAX_RESULTS then
						for i = 1, MAX_RESULTS do reply(tbl[i], player) end
						for i = 1, MAX_RESULTS do table.remove(tbl, 1) end
						reply(TOO_MANY:format(#tbl), player)
						moreResults[player] = tbl
					else
						for i, v in ipairs(tbl) do reply(v, player) end
						tbl = nil
					end
				elseif isWhisper then
					reply(NO_DATA:format(query), player)
				end
			else
				updateAcceptedCommands()
				reply(NO_SKILL:format(table.concat(acceptedCommands, ", ")), player)
			end
		elseif isWhisper then
			reply(CG_USAGE:format(cmd), player)
		end
	end
	function addon:CHAT_MSG_GUILD(msg, player)
		processChat(msg, player)
	end
	function addon:CHAT_MSG_WHISPER(msg, player)
		processChat(msg, player, true)
	end
end

function addon:PLAYER_ENTERING_WORLD()
	if not CraftyGnomeDB then CraftyGnomeDB = {} end
	local realm = GetRealmName()
	if not CraftyGnomeDB[realm] then CraftyGnomeDB[realm] = {} end
	local faction = UnitFactionGroup("player")
	if not CraftyGnomeDB[realm][faction] then CraftyGnomeDB[realm][faction] = {} end
	db = CraftyGnomeDB[realm][faction]

	if not db.version or db.version < 3 then
		for k in pairs(db) do db[k] = nil end
		db.version = 3
	end

	addon:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

local orig = ChatFrame_MessageEventHandler
local chatPrefixTest = "^" .. PREFIX
ChatFrame_MessageEventHandler = function(event, ...)
	if event == "CHAT_MSG_WHISPER" then
		local cmd = select(3, arg1:find("^!(%S+)%s"))
		if bangMappings[cmd] then return end
	elseif event == "CHAT_MSG_WHISPER_INFORM" and arg1:find(chatPrefixTest) then
		return
	end
	return orig(event, ...)
end

addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
addon:RegisterEvent("PLAYER_ENTERING_WORLD")
addon:RegisterEvent("CHAT_MSG_GUILD")
addon:RegisterEvent("CHAT_MSG_WHISPER")
addon:RegisterEvent("CRAFT_SHOW")
addon:RegisterEvent("TRADE_SKILL_SHOW")

